@@ -28,8 +28,10 @@ from ha_tools import use_greenlets
 
 from pymongo import (ReplicaSetConnection,
                      ReadPreference)
-from pymongo.replica_set_connection import Member, Monitor
-from pymongo.connection import Connection, _partition_node
+from pymongo.read_preferences import modes
+from pymongo.mongo_replica_set_client import Member, Monitor
+from pymongo.mongo_client import _partition_node
+from pymongo.connection import Connection
 from pymongo.errors import AutoReconnect, OperationFailure
 
 from test import utils
@@ -66,8 +68,7 @@ class TestDirectConnection(unittest.TestCase):
         arbiter_host, arbiter_port = ha_tools.get_arbiters()[0].split(':')
         arbiter_port = int(arbiter_port)
 
-        # A Connection succeeds no matter the read preference (although a
-        # secondary Connection with preference PRIMARY can't be queried)
+        # A Connection succeeds no matter the read preference
         for kwargs in [
             {'read_preference': ReadPreference.PRIMARY},
             {'read_preference': ReadPreference.PRIMARY_PREFERRED},
@@ -84,8 +85,8 @@ class TestDirectConnection(unittest.TestCase):
             self.assertEqual(primary_port, conn.port)
             self.assertTrue(conn.is_primary)
 
-            if kwargs.get('read_preference') != ReadPreference.SECONDARY:
-                self.assertTrue(conn.pymongo_test.test.find_one())
+            # Direct connection to primary can be queried with any read pref
+            self.assertTrue(conn.pymongo_test.test.find_one())
 
             conn = Connection(secondary_host,
                               secondary_port,
@@ -95,6 +96,8 @@ class TestDirectConnection(unittest.TestCase):
             self.assertEqual(secondary_port, conn.port)
             self.assertFalse(conn.is_primary)
 
+            # Direct connection to secondary can be queried with any read pref
+            # but PRIMARY
             if kwargs.get('read_preference') != ReadPreference.PRIMARY:
                 self.assertTrue(conn.pymongo_test.test.find_one())
             else:
