@@ -253,6 +253,18 @@ class TestConnection(unittest.TestCase, TestRequestMixin):
         self.assertEqual(c, Connection("mongodb://%s:%d" %
                                        (self.host, self.port)))
 
+        self.assertTrue(Connection("mongodb://%s:%d" %
+                                (self.host, self.port),
+                                slave_okay=True).slave_okay)
+        self.assertTrue(Connection("mongodb://%s:%d/?slaveok=true;w=2" %
+                                (self.host, self.port)).slave_okay)
+
+    def test_auth_from_uri(self):
+        c = Connection(self.host, self.port)
+        # Sharded auth not supported before MongoDB 2.0
+        if is_mongos(c) and not version.at_least(c, (2, 0, 0)):
+            raise SkipTest("Auth with sharding requires MongoDB >= 2.0.0")
+
         c.admin.system.users.remove({})
         c.pymongo_test.system.users.remove({})
         c.admin.add_user("admin", "pass")
@@ -276,11 +288,6 @@ class TestConnection(unittest.TestCase, TestRequestMixin):
         Connection("mongodb://user:pass@%s:%d/pymongo_test" %
                    (self.host, self.port))
 
-        self.assertTrue(Connection("mongodb://%s:%d" %
-                                (self.host, self.port),
-                                slave_okay=True).slave_okay)
-        self.assertTrue(Connection("mongodb://%s:%d/?slaveok=true;w=2" %
-                                (self.host, self.port)).slave_okay)
         c.admin.system.users.remove({})
         c.pymongo_test.system.users.remove({})
 
@@ -398,6 +405,25 @@ class TestConnection(unittest.TestCase, TestRequestMixin):
         conn = Connection(self.host, self.port, socketTimeoutMS=10500)
         self.assertEqual(10.5, conn._MongoClient__pool.net_timeout)
 
+    def test_network_timeout_validation(self):
+        c = get_connection(network_timeout=10)
+        self.assertEqual(10, c._MongoClient__net_timeout)
+
+        c = get_connection(network_timeout=None)
+        self.assertEqual(None, c._MongoClient__net_timeout)
+
+        self.assertRaises(ConfigurationError,
+            get_connection, network_timeout=0)
+
+        self.assertRaises(ConfigurationError,
+            get_connection, network_timeout=-1)
+
+        self.assertRaises(ConfigurationError,
+            get_connection, network_timeout=1e10)
+
+        self.assertRaises(ConfigurationError,
+            get_connection, network_timeout='foo')
+
     def test_network_timeout(self):
         no_timeout = Connection(self.host, self.port)
         timeout_sec = 1
