@@ -23,6 +23,7 @@ from nose.plugins.skip import SkipTest
 sys.path[0:0] = [""]
 
 from bson.son import SON
+from pymongo.cursor import _QUERY_OPTIONS
 from pymongo.replica_set_connection import ReplicaSetConnection
 from pymongo.read_preferences import (ReadPreference, modes, MovingAverage,
                                       secondary_ok_commands)
@@ -76,6 +77,58 @@ class TestReadPreferencesBase(TestConnectionReplicaSetBase):
 
 
 class TestReadPreferences(TestReadPreferencesBase):
+    def test_mode_validation(self):
+        # 'modes' are imported from read_preferences.py
+        for mode in modes:
+            self.assertEqual(mode, self._get_connection(
+                read_preference=mode).read_preference)
+
+        self.assertRaises(ConfigurationError, self._get_connection,
+            read_preference='foo')
+
+    def test_tag_sets_validation(self):
+        # Can't use tags with PRIMARY
+        self.assertRaises(ConfigurationError, self._get_connection,
+            tag_sets=[{'k': 'v'}])
+
+        # ... but empty tag sets are ok with PRIMARY
+        self.assertEqual([{}], self._get_connection(tag_sets=[{}]).tag_sets)
+
+        S = ReadPreference.SECONDARY
+        self.assertEqual([{}], self._get_connection(read_preference=S).tag_sets)
+
+        self.assertEqual([{'k': 'v'}], self._get_connection(
+            read_preference=S, tag_sets=[{'k': 'v'}]).tag_sets)
+
+        self.assertEqual([{'k': 'v'}, {}], self._get_connection(
+            read_preference=S, tag_sets=[{'k': 'v'}, {}]).tag_sets)
+
+        self.assertRaises(ConfigurationError, self._get_connection,
+            read_preference=S, tag_sets=[])
+
+        # One dict not ok, must be a list of dicts
+        self.assertRaises(ConfigurationError, self._get_connection,
+            read_preference=S, tag_sets={'k': 'v'})
+
+        self.assertRaises(ConfigurationError, self._get_connection,
+            read_preference=S, tag_sets='foo')
+
+        self.assertRaises(ConfigurationError, self._get_connection,
+            read_preference=S, tag_sets=['foo'])
+
+    def test_latency_validation(self):
+        self.assertEqual(17, self._get_connection(
+            secondary_acceptable_latency_ms=17
+        ).secondary_acceptable_latency_ms)
+
+        self.assertEqual(42, self._get_connection(
+            secondaryAcceptableLatencyMS=42
+        ).secondary_acceptable_latency_ms)
+
+        self.assertEqual(666, self._get_connection(
+            secondaryacceptablelatencyms=666
+        ).secondary_acceptable_latency_ms)
+
     def test_primary(self):
         self.assertReadsFrom('primary',
             read_preference=ReadPreference.PRIMARY)
@@ -428,35 +481,68 @@ class TestMongosConnection(unittest.TestCase):
         # Test default mode, PRIMARY
         cursor = c.pymongo_test.test.find()
         if is_mongos:
+            # We only set $readPreference if it's something other than
+            # PRIMARY to avoid problems with mongos versions that don't
+            # support read preferences.
             self.assertEqual(
-                {'mode': 'primary'},
+                None,
                 cursor._Cursor__query_spec().get('$readPreference')
             )
         else:
             self.assertFalse(
                 '$readPreference' in cursor._Cursor__query_spec())
 
+        # Copy these constants for brevity
+        PRIMARY_PREFERRED = ReadPreference.PRIMARY_PREFERRED
+        SECONDARY = ReadPreference.SECONDARY
+        SECONDARY_PREFERRED = ReadPreference.SECONDARY_PREFERRED
+        NEAREST = ReadPreference.NEAREST
+        SLAVE_OKAY = _QUERY_OPTIONS['slave_okay']
+
         # Test non-PRIMARY modes which can be combined with tags
-        for mode, mongos_mode in (
-            (ReadPreference.PRIMARY_PREFERRED, 'primaryPreferred'),
-            (ReadPreference.SECONDARY, 'secondary'),
-            (ReadPreference.SECONDARY_PREFERRED, 'secondaryPreferred'),
-            (ReadPreference.NEAREST, 'nearest'),
+        for kwarg, value, mongos_mode in (
+            ('read_preference', PRIMARY_PREFERRED, 'primaryPreferred'),
+            ('read_preference', SECONDARY, 'secondary'),
+            ('read_preference', SECONDARY_PREFERRED, 'secondaryPreferred'),
+            ('read_preference', NEAREST, 'nearest'),
+            ('slave_okay', True, 'secondaryPreferred'),
+            ('slave_okay', False, 'primary')
         ):
             for tag_sets in (
                 None, [{}]
             ):
-                c = get_connection(
-                    read_preference=mode,
-                    tag_sets=tag_sets)
+                # Create a connection e.g. with read_preference=NEAREST or
+                # slave_okay=True
+                c = get_connection(tag_sets=tag_sets, **{kwarg: value})
 
                 self.assertEqual(is_mongos, c.is_mongos)
                 cursor = c.pymongo_test.test.find()
                 if is_mongos:
-                    self.assertEqual(
-                        {'mode': mongos_mode},
-                        cursor._Cursor__query_spec().get('$readPreference')
-                    )
+                    # We don't set $readPreference for SECONDARY_PREFERRED
+                    # unless tags are in use. slaveOkay has the same effect.
+                    if mongos_mode == 'secondaryPreferred':
+                        self.assertEqual(
+                            None,
+                            cursor._Cursor__query_spec().get('$readPreference'))
+
+                        self.assertTrue(
+                            cursor._Cursor__query_options() & SLAVE_OKAY)
+
+                    # Don't send $readPreference for PRIMARY either
+                    elif mongos_mode == 'primary':
+                        self.assertEqual(
+                            None,
+                            cursor._Cursor__query_spec().get('$readPreference'))
+
+                        self.assertFalse(
+                            cursor._Cursor__query_options() & SLAVE_OKAY)
+                    else:
+                        self.assertEqual(
+                            {'mode': mongos_mode},
+                            cursor._Cursor__query_spec().get('$readPreference'))
+
+                        self.assertTrue(
+                            cursor._Cursor__query_options() & SLAVE_OKAY)
                 else:
                     self.assertFalse(
                         '$readPreference' in cursor._Cursor__query_spec())
@@ -466,9 +552,16 @@ class TestMongosConnection(unittest.TestCase):
                 [{'dc': 'la'}, {'dc': 'sf'}],
                 [{'dc': 'la'}, {'dc': 'sf'}, {}],
             ):
-                c = get_connection(
-                    read_preference=mode,
-                    tag_sets=tag_sets)
+                if kwarg == 'slave_okay':
+                    # Can't use tags with slave_okay True or False, need a
+                    # real read preference
+                    self.assertRaises(
+                        ConfigurationError,
+                        get_connection, tag_sets=tag_sets, **{kwarg: value})
+
+                    continue
+
+                c = get_connection(tag_sets=tag_sets, **{kwarg: value})
 
                 self.assertEqual(is_mongos, c.is_mongos)
                 cursor = c.pymongo_test.test.find()
@@ -492,12 +585,16 @@ class TestMongosConnection(unittest.TestCase):
                 continue
             command = SON([(cmd, 1)])
             cursor = c.pymongo_test["$cmd"].find(command.copy())
+            # White-listed commands also have to be wrapped in $query
+            command = SON([('$query', command)])
             command['$readPreference'] = {'mode': 'secondary'}
             self.assertEqual(command, cursor._Cursor__query_spec())
 
         # map_reduce inline should have read prefs
         command = SON([('mapreduce', 'test'), ('out', {'inline': 1})])
         cursor = c.pymongo_test["$cmd"].find(command.copy())
+        # White-listed commands also have to be wrapped in $query
+        command = SON([('$query', command)])
         command['$readPreference'] = {'mode': 'secondary'}
         self.assertEqual(command, cursor._Cursor__query_spec())
 
